// system headers
#include <stdio.h>

// global FaceCell SDK headers
#include <NCore.h>
#include <NImages.h>
#include <FccExtractor.h>

char title[] = "FaceCell Find Eyes Tutorial, Version 1.0.0.0, Copyright (C) 2006-2007 Neurotechnologija\n";

int usage()
{
	printf("usage:\n");
	printf("\tFindEyes [Image]\n");
	printf("\n");
	printf("\t[Image] - filename of image.\n");

	return 1;
}

int main(int argc, char **argv)
{
	NResult result;
	HFccExtractor extractor;

	HNImage image, grayscale;

	int faceCount;
	FcceFace *faces;
	FcceEyes eyes;

	int i;

	printf(title);
	if (argc < 2)
	{
		return usage();
	}

	// read image
	result = NImageCreateFromFile(argv[1], NULL, &image);
	if (NFailed(result))
	{
		printf("image loading failed (result = %d)!\n", result);
		return -1;
	}

	// convert image to grayscale
	result = NImageCreateFromImage(npfGrayscale, 0, image, &grayscale);
	NImageFree(image);
	if (NFailed(result))
	{
		printf("image conversion to grayscale failed (result = %d)!\n", result);
		return -1;
	}

	// create an extractor
	result = FcceCreate(&extractor);
	if (NFailed(result))
	{
		printf("extractor creation failed (result = %d)!\n", result);
		NImageFree(grayscale);
		return -1;
	}

	// detect all faces that are suitable for face recognition in the image
	result = FcceDetectFaces(extractor, grayscale, &faceCount, &faces);
	if (NFailed(result))
	{
		printf("face detection failed (result = %d)!\n", result);

		NImageFree(grayscale);
		FcceFree(extractor);
		return -1;
	}
	printf("found faces:\n");
	for (i = 0; i < faceCount; ++i)
	{
		printf("\tlocation = (%d, %d), width = %d, height = %d, confidence = %.2f\n",
			faces[i].Rectangle.X, faces[i].Rectangle.Y,
			faces[i].Rectangle.Width, faces[i].Rectangle.Height,
			faces[i].Confidence);

		// detect eyes for current face
		result = FcceDetectEyes(extractor, grayscale, &faces[i], &eyes);
		if (NFailed(result))
		{
			printf("\t\teye detectiond failed (result = %d)!\n", result);
		}
		else
		{
			printf("\tfound eyes:\n");
			printf("\t\tfirst: location = (%d, %d), confidence = %.2f\n",
				eyes.First.X, eyes.First.Y, eyes.FirstConfidence);
			printf("\t\tsecond: location = (%d, %d), confidence = %.2f\n",
				eyes.Second.X, eyes.Second.Y, eyes.SecondConfidence);
		}
	}

	// free resources
	NImageFree(grayscale);
	FcceFree(extractor);
	NFree(faces);

	return 0;
}
